@@ -16,32 +16,26 @@ pub fn generate<T: Service>(service: &T, proto_path: &str) -> TokenStream {
     let service_doc = generate_doc_comments(service.comment());
     let (serve_filters, serve_combined_filters) = generate_filters(service, proto_path);
 
-    let filters_method = {
-        #[cfg(feature = "boxed")]
-        quote! {
-            /// Convert this service to `warp` `Filter`s.
-            ///
-            /// This can be used to compose multiple services. See `serve_multiple` macro in `hrpc`.
-            #[allow(clippy::redundant_clone)]
-            pub fn filters(self) -> BoxedFilter<(impl Reply,)> {
-                let server = self.inner;
+    let filters_method = quote! {
+        /// Convert this service to `warp` `Filter`s.
+        ///
+        /// This can be used to compose multiple services. See `serve_multiple` macro in `hrpc`.
+        #[allow(clippy::redundant_clone)]
+        #[cfg(not(debug_assertions))]
+        pub fn filters(self) -> impl Filter<Extract = (impl Reply,), Error = warp::Rejection> + Clone {
+            let server = self.inner;
 
-                #serve_filters
-                #serve_combined_filters.boxed()
-            }
+            #serve_filters
+            balanced_or_tree!(#serve_combined_filters)
         }
-        #[cfg(not(feature = "boxed"))]
-        quote! {
-            /// Convert this service to `warp` `Filter`s.
-            ///
-            /// This can be used to compose multiple services. See `serve_multiple` macro in `hrpc`.
-            #[allow(clippy::redundant_clone)]
-            pub fn filters(self) -> impl Filter<Extract = (impl Reply,), Error = warp::Rejection> + Clone {
-                let server = self.inner;
 
-                #serve_filters
-                #serve_combined_filters
-            }
+        #[allow(clippy::redundant_clone)]
+        #[cfg(debug_assertions)]
+        pub fn filters(self) -> BoxedFilter<(impl Reply,)> {
+            let server = self.inner;
+
+            #serve_filters
+            balanced_or_tree!(#serve_combined_filters).boxed()
         }
     };
 
@@ -177,7 +171,7 @@ fn generate_filters<T: Service>(service: &T, _proto_path: &str) -> (TokenStream,
     let mut stream = TokenStream::new();
     let mut comb_stream = TokenStream::new();
 
-    for (index, method) in service.methods().iter().enumerate() {
+    for method in service.methods().iter() {
         let name = quote::format_ident!("{}", method.name());
         let on_upgrade_response_name = quote::format_ident!("{}_on_upgrade", name);
         let pre_name = quote::format_ident!("{}_middleware", name);
@@ -246,33 +240,7 @@ fn generate_filters<T: Service>(service: &T, _proto_path: &str) -> (TokenStream,
             };
         };
 
-        comb_stream.extend(if index > 0 {
-            #[cfg(feature = "boxed")]
-            {
-                quote! {
-                    .or(#name.boxed())
-                }
-            }
-            #[cfg(not(feature = "boxed"))]
-            {
-                quote! {
-                    .or(#name)
-                }
-            }
-        } else {
-            #[cfg(feature = "boxed")]
-            {
-                quote! {
-                    #name.boxed()
-                }
-            }
-            #[cfg(not(feature = "boxed"))]
-            {
-                quote! {
-                    #name
-                }
-            }
-        });
+        comb_stream.extend(quote! { #name, });
 
         stream.extend(apply_middleware);
     }
